/*
 *  qm_tracks.cpp
 *  QUIMUP library & playlist window
 *  (c) 2008-2010 Johan Spee
 *
 *  This file is part of Quimup
 *
 *  QUIMUP is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  QUIMUP is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see http://www.gnu.org/licenses/.
 */
 
#include "qm_tracks.h"

qm_tracks::qm_tracks()
{ 
	setupUI();
	set_local_features(false);
}


void qm_tracks::setupUI()
{
	if (objectName().isEmpty())
		setObjectName("tracksWin");
      
	QPixmap qpx;
	qpx = QPixmap(":/mn_icon.png");
	setWindowIcon(qpx);
	setWindowTitle(trUtf8("Quimup Playlist"));
		
    QSizePolicy sizePolicy(static_cast<QSizePolicy::Policy>(5), static_cast<QSizePolicy::Policy>(5));
    sizePolicy.setHorizontalStretch(0);
    sizePolicy.setVerticalStretch(0);
    sizePolicy.setHeightForWidth(this->sizePolicy().hasHeightForWidth());
    setSizePolicy(sizePolicy);
    setMinimumSize(QSize(600, 400));
    setMaximumSize(QSize(32767, 32767));
		
    theSplitter = new QSplitter(this);
	setCentralWidget(theSplitter); // 'attatches' it to the window
    theSplitter->setOrientation(Qt::Horizontal);
		
	left_group = new QGroupBox();
	left_group->setAlignment(Qt::AlignHCenter);
		vbox_lefthalf = new QVBoxLayout(left_group);
		vbox_lefthalf->setSpacing(4);
		vbox_lefthalf->setMargin(6);
			hbox_leftop = new QHBoxLayout();
			hbox_leftop->setSpacing(4);
			hbox_leftop->setMargin(0);
				cb_dbasemode = new QComboBox();
				cb_dbasemode->setMinimumSize(QSize(140, 26));
			hbox_leftop->addWidget(cb_dbasemode);	
				lb_title_topleft = new QLabel();
				lb_title_topleft->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
				lb_title_topleft->setAlignment(Qt::AlignHCenter);
				lb_title_topleft->setTextFormat(Qt::RichText);
			hbox_leftop->addWidget(lb_title_topleft);	
				bt_update = new QPushButton();
				bt_update->setMinimumSize(QSize(120, 26));
				bt_update->setIcon(QIcon(":/tr_update.png"));
			hbox_leftop->addWidget(bt_update);
		vbox_lefthalf->addLayout(hbox_leftop);
			lib_view = new qm_libview(); 
		vbox_lefthalf->addWidget(lib_view);
			lb_dbasestats = new QLabel();
			lb_dbasestats->setAlignment(Qt::AlignHCenter);
		vbox_lefthalf->addWidget(lb_dbasestats);
			hbox_leftbottom = new QHBoxLayout();
				cb_searchmode = new QComboBox();
				cb_searchmode->setMinimumSize(QSize(120, 26));
			hbox_leftbottom->addWidget(cb_searchmode);
				le_search = new QLineEdit();
			hbox_leftbottom->addWidget(le_search);
				bt_search = new QPushButton();
				bt_search->setMinimumSize(QSize(120, 26));
				bt_search->setIcon(QIcon(":/tr_pointleft.png"));
			hbox_leftbottom->addWidget(bt_search);
		vbox_lefthalf->addLayout(hbox_leftbottom);	
	theSplitter->addWidget(left_group);
			
	right_group = new QGroupBox();
	right_group->setAlignment(Qt::AlignHCenter);
		vbox_righthalf = new QVBoxLayout(right_group);
		vbox_righthalf->setSpacing(4);
		vbox_righthalf->setMargin(6);
			hbox_rightop = new QHBoxLayout();
			hbox_rightop->setSpacing(4);
			hbox_rightop->setMargin(0);
				bt_stream = new QPushButton();
				bt_stream->setMinimumSize(QSize(120, 26));
				bt_stream->setIcon(QIcon(":/tr_stream.png"));
			hbox_rightop->addWidget(bt_stream);
				lb_title_topright = new QLabel();
				lb_title_topright->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
				lb_title_topright->setAlignment(Qt::AlignHCenter);
				lb_title_topright->setTextFormat(Qt::RichText);
			hbox_rightop->addWidget(lb_title_topright);	
				bt_options = new QPushButton();
				bt_options->setMinimumSize(QSize(120, 26));
				bt_options->setIcon(QIcon(":/tr_options.png"));
			hbox_rightop->addWidget(bt_options);	
		vbox_righthalf->addLayout(hbox_rightop);
			plist_view = new qm_plistview(); 
		vbox_righthalf->addWidget(plist_view);
			lb_pliststats = new QLabel();
			lb_pliststats->setAlignment(Qt::AlignHCenter);
		vbox_righthalf->addWidget(lb_pliststats);
			hbox_rightbottom = new QHBoxLayout();
				cb_selectmode = new QComboBox();
				cb_selectmode->setMinimumSize(QSize(120, 26));
			hbox_rightbottom->addWidget(cb_selectmode);
				le_select = new QLineEdit();
			hbox_rightbottom->addWidget(le_select);
				bt_select = new QPushButton();
				bt_select->setMinimumSize(QSize(120, 26));
				bt_select->setIcon(QIcon(":/tr_pointleft.png"));
			hbox_rightbottom->addWidget(bt_select);
		vbox_righthalf->addLayout(hbox_rightbottom);	
	theSplitter->addWidget(right_group);

	cb_dbasemode->addItem ( QIcon(":/tr_artist.png"), trUtf8(" Artists "));
	cb_dbasemode->addItem ( QIcon(":/tr_album.png"), trUtf8(" Albums "));
	cb_dbasemode->addItem ( QIcon(":/tr_genre.png"), trUtf8(" Genres "));
	cb_dbasemode->addItem ( QIcon(":/tr_playlist.png"), trUtf8(" Playlists "));
	cb_dbasemode->addItem ( QIcon(":/tr_folder.png"), trUtf8(" Folders "));
	cb_dbasemode->addItem ( QIcon(":/tr_foldate.png"), trUtf8(" Sorted by date "));
	cb_dbasemode->addItem ( QIcon(":/tr_search.png"), trUtf8(" Search results "));
	cb_dbasemode->setIconSize(QSize(16, 16));
	
	cb_searchmode->insertItems(0, QStringList()
        << trUtf8(" Artist")
		<< trUtf8(" Album")
		<< trUtf8(" Title")
        << trUtf8(" Genre") );
	 
	cb_selectmode->insertItems(0, QStringList()
		<< trUtf8(" Artist")
		<< trUtf8(" Album")
		<< trUtf8(" Title")
		<< trUtf8(" All") );

	sloader = new qm_stream_loader();
	
	options_menu = new QMenu();
	
	a_random = new QAction(options_menu);
    a_random->setText(trUtf8("Random playback"));
	a_random->setCheckable(true);
	a_random->setIcon(QIcon(":/tr_random.png"));
    QObject::connect( a_random, SIGNAL( triggered() ), this, SLOT( set_random() ) );
	options_menu->addAction(a_random);
	
	a_repeat = new QAction(options_menu);
    a_repeat->setText(trUtf8("Repeat (list or single)")); 
	a_repeat->setCheckable(true);
	a_repeat->setIcon(QIcon(":/tr_repeat.png"));
    QObject::connect( a_repeat, SIGNAL( triggered() ), this, SLOT( set_repeat() ) );
	options_menu->addAction(a_repeat);

	a_single = new QAction(options_menu);
    a_single->setText(trUtf8("Single mode"));
	a_single->setCheckable(true);
	a_single->setIcon(QIcon(":/tr_single.png"));
    QObject::connect( a_single, SIGNAL( triggered() ), this, SLOT( set_single() ) );
	options_menu->addAction(a_single);
	
	a_consume = new QAction(options_menu);
    a_consume->setText(trUtf8("Consume mode"));
	a_consume->setCheckable(true);
	a_consume->setIcon(QIcon(":/tr_consume.png"));
    QObject::connect( a_consume, SIGNAL( triggered() ), this, SLOT( set_consume() ) );
	options_menu->addAction(a_consume);
	
	a_xfade = new QAction(options_menu);
    a_xfade->setText(trUtf8("Cross fade"));
	a_xfade->setCheckable(true);
	a_xfade->setIcon(QIcon(":/tr_xfade.png"));
    QObject::connect( a_xfade, SIGNAL( triggered() ), this, SLOT( set_xfade() ) );
	options_menu->addAction(a_xfade);

	QObject::connect(sloader, SIGNAL(streamdownload_done(QStringList)), this, SLOT(on_streamdownload_done(QStringList)));	
	QObject::connect(cb_dbasemode, SIGNAL(activated(int)), this, SLOT(on_dbase_newmode(int)));
    QObject::connect(cb_searchmode, SIGNAL(activated(int)), this, SLOT(set_searchmode(int)));
    QObject::connect(cb_selectmode, SIGNAL(activated(int)), this, SLOT(set_selectmode(int)));	
	QObject::connect(bt_search, SIGNAL(clicked()), this, SLOT(search_in_lib()));
    QObject::connect(le_search, SIGNAL(returnPressed()), this, SLOT(search_in_lib()));
	QObject::connect(bt_select, SIGNAL(clicked()), this, SLOT(select_in_plist()));
    QObject::connect(le_select, SIGNAL(returnPressed()), this, SLOT(select_in_plist()));
    QObject::connect(bt_update, SIGNAL(clicked()), this, SLOT(on_update_dbase()));	
    QObject::connect(bt_stream, SIGNAL(clicked()), this, SLOT(get_the_stream()));
    QObject::connect(bt_options, SIGNAL(clicked()), this, SLOT(on_options_clicked()));
	QObject::connect(plist_view, SIGNAL(sgnl_plist_statistics(QString)), this, SLOT(set_plist_stats(QString)));
	QObject::connect(plist_view, SIGNAL(sgnl_plist_dropreceived(int)), this, SLOT(on_plist_dropreceived(int)));
	QObject::connect(lib_view, SIGNAL(sgnl_newmode(int)), this, SLOT(on_lib_newmode(int)));
	QObject::connect(lib_view, SIGNAL(sgnl_playlist_modification(bool)), this, SLOT(on_playlist_modification(bool)));
	QObject::connect(theSplitter, SIGNAL(splitterMoved(int,int)), this, SLOT(splittermoved()));

/* 	plist_view and lib_view share thePlaylistModList
	for playlist modifications like drag&drop operations. */	
	lib_view->set_playlistModList( &thePlaylistModList );
	plist_view->set_playlistModList( &thePlaylistModList );

} // setupUi


void qm_tracks::get_the_stream()
{
	QString suggested_name = "http://";
	bool ok;
    QString input = QInputDialog::getText(this, trUtf8("Save"),
                                          trUtf8(" URL for the stream's playlist file: "), QLineEdit::Normal,
                                          suggested_name, &ok);
    if ( !ok || input.isEmpty() || input == "http://")
    return;
	
	if (!input.startsWith("http://"))
	{
		QMessageBox::warning( this, trUtf8("Error"), "URL must begin with \"http://\"");
		get_the_stream();
	}
	
	sloader->download_this(input);
	// and wait for sgnl_on_streamdownload_done
}


void qm_tracks::on_streamdownload_done(QStringList result)
{
	thePlaylistModList.clear();
	if (result.value(0) == "error")
	{
		QMessageBox::warning( this, trUtf8("Error"), result.value(1));
		return;
	}
	
	if (result.value(0) == "nothing")
	{
		QString message = trUtf8("Expected playlist file was not found.\nDownload aborted. ");
		QMessageBox::warning( this, trUtf8("Error"), message);
		return;
	}
	
	
	if (result.value(0) == "timeout")
	{
		QString message = trUtf8("Download takes too long (") + result.value(1) + trUtf8(" % in 10 sec.)\nDownload aborted. ");
		QMessageBox::warning( this, trUtf8("Error"), message);
		return;
	}
	
	int count = 0;
	
	for (int i = 2; i < result.size(); ++i)
	{
		qm_mpd_command newCommand;
		newCommand.cmd = CMD_ADD;
		newCommand.file = result.value(i);
		newCommand.moveTo = 0;
		newCommand.artist = "Stream ";
		newCommand.title = result.value(i);
		newCommand.time = 0;
		newCommand.album = "Info in player ";
		newCommand.track = "";
		newCommand.type = TP_STREAM;
		thePlaylistModList.push_back(newCommand);
		count++;
	}
	
	mpdCom->execute_cmds(thePlaylistModList, false );
	plist_view->on_playlist_modification(false);
	
	QString message = "Downloaded file type: " + result.value(1) + " \nAdded " + QString::number(count) + " items to the playlist. ";
	
	QMessageBox::information( this, trUtf8("Confirmation"), message);
}


void qm_tracks::on_playlist_modification(bool dragdrop)
{
	plist_view->on_playlist_modification(dragdrop);
}

void qm_tracks::on_plist_dropreceived(int pos)
{
	lib_view->on_plist_dropreceived(pos);
}

void qm_tracks::on_lib_newmode(int newmode)
{
	cb_dbasemode->blockSignals(true);
	cb_dbasemode->setCurrentIndex(newmode);
	cb_dbasemode->blockSignals(false);
	// libview must know what to search for
	if (newmode == LB_ID_SEARCH)
		config->search_string = le_search->text();
}

void qm_tracks::on_dbase_newmode(int i)
{

	config->tracks_dbasemode = i;		
	lib_view->reload_root();	
}


void qm_tracks::set_searchmode(int i)
{
	config->tracks_searchmode = i;
}


void qm_tracks::set_selectmode(int i)
{
	config->tracks_selectmode = i;	
}


void qm_tracks::search_in_lib()
{
	config->search_string = le_search->text();
	cb_dbasemode->blockSignals(true);
	cb_dbasemode->setCurrentIndex(LB_ID_SEARCH);
	cb_dbasemode->blockSignals(false);
	on_dbase_newmode(LB_ID_SEARCH);
	lib_view->setFocus();
}


void qm_tracks::select_in_plist()
{
	config->select_string = le_select->text();
	plist_view->select_it(cb_selectmode->currentIndex() );
	plist_view->setFocus();
}


void qm_tracks::on_options_clicked()
{
  options_menu->exec(QCursor::pos());
}

void qm_tracks::retranslateUi()
{
	if (config->show_tooltips)
	{
		bt_stream->setToolTip(trUtf8(" Enter a URL to an internet audio stream "));
		bt_options->setToolTip(trUtf8(" Set playback options "));
		bt_update->setToolTip(trUtf8(" Scan the entire music collection for changes "));
		cb_searchmode->setToolTip(trUtf8(" Search mode "));
		cb_selectmode->setToolTip(trUtf8(" Select mode "));
		cb_dbasemode->setToolTip(trUtf8(" Library mode "));
	}
	
	lb_title_topleft->setText(trUtf8("<b>Library</b>"));
	lb_title_topright->setText(trUtf8("<b>Playlist</b>"));
	bt_select->setText(trUtf8("Select"));
	bt_search->setText(trUtf8("Search"));
	bt_update->setText(trUtf8("Update"));
	bt_options->setText(trUtf8("Playback options"));
	bt_stream->setText(trUtf8("Add Stream"));
}


void qm_tracks::set_config(qm_config *conf)
{
	if (conf == NULL)
		return;

	config = conf;
	retranslateUi();
	cb_dbasemode->blockSignals(true);
	cb_dbasemode->setCurrentIndex(config->tracks_dbasemode);
	cb_dbasemode->blockSignals(false);
	cb_searchmode->setCurrentIndex(config->tracks_searchmode);
	if (config->tracks_dbasemode != LB_ID_SEARCH)
		config->search_string = "";
	le_search->setText(config->search_string);	
	cb_selectmode->setCurrentIndex(config->tracks_selectmode);
	config->select_string = "";
	le_select->setText(config->select_string);
	
    // set the splitter pos
    QList<int> List;
    List.append(config->tracks_L_splitter);
    List.append(config->tracks_R_splitter);
    theSplitter->setSizes(List);
	
	resize(config->tracksWin_W, config->tracksWin_H );
}


void qm_tracks::set_font(QFont & thefont)
{
	plist_view->set_font(thefont);
	lib_view->set_font(thefont);
}


void qm_tracks::splittermoved()
{
	QList<int> list = theSplitter->sizes();
	config->tracks_L_splitter = list.value(0);
	config->tracks_R_splitter = list.value(1);
}


void qm_tracks::hideEvent(QHideEvent *he)
{
	config->tracksWin_X = x();
	config->tracksWin_Y = y();
	config->tracksWin_W = width();
	config->tracksWin_H = height();
	he->accept();
}


void qm_tracks::set_plist_stats(QString stats)
{
	lb_pliststats->setText(stats);
}


void qm_tracks::set_dbase_stats(QString stats)
{
	lb_dbasestats->setText(stats);
}


void qm_tracks::showEvent(QShowEvent *se)
{
	move(config->tracksWin_X, config->tracksWin_Y);
	se->accept();
}


void qm_tracks::closeEvent( QCloseEvent *ce )
{
	this->hide();
    ce->ignore(); // only hide, don't close
}


void qm_tracks::set_mpdCom(qm_mpdCom* conn, bool isconnected)
{
	mpd_connected = isconnected;
    mpdCom = conn;
	QObject::connect(mpdCom, SIGNAL( sgnl_dbase_statistics(QString)), this, SLOT(set_dbase_stats(QString)));
	QObject::connect(mpdCom, SIGNAL( sgnl_update_ready()), this, SLOT(on_update_ready()));
	mpdCom->get_statistics();
	
	if (!config->mpd_update_allowed)
		bt_update->setEnabled(false);
	else
		bt_update->setEnabled(true);
		
	if (config->mpd_single_allowed)
		a_single->setEnabled(true);
	else
		a_single->setEnabled(false);
		
	if (config->mpd_consume_allowed)
		a_consume->setEnabled(true);
	else
		a_consume->setEnabled(false);
		
	if (config->mpd_xfade_allowed)
		a_xfade->setEnabled(true);
	else
		a_xfade->setEnabled(false);	
		
	if (config->mpd_repeat_allowed)
		a_repeat->setEnabled(true);
	else
		a_repeat->setEnabled(false);
		
	if (config->mpd_random_allowed)
		a_random->setEnabled(true);
	else
		a_random->setEnabled(false);
		
	if (mpd_connected)
		set_local_features(config->mpd_local_features);
	else
		set_local_features(false);
}


void qm_tracks::on_update_dbase()
{
	if (!config->mpd_update_allowed)
		return;
		
	bt_update->setEnabled(false);
	bt_update->setIcon(QIcon(":/tr_dot.png"));
	mpdCom->update_all();
}


void qm_tracks::on_update_ready()
{
	bt_update->setDisabled(false);
	bt_update->setIcon(QIcon(":/tr_update.png"));
	lib_view->reload_root();
}


void qm_tracks::set_colors()
{
	QPalette pal (this->palette());
	
	if (config->colormainonly)
	{
		lb_title_topleft->setPalette(pal);
		lb_title_topright->setPalette(pal);
		lb_pliststats->setPalette(pal);
		lb_dbasestats->setPalette(pal);
		return;
	}
	
	QString foregdcol;

	if (config->rb_qmcool_checked)
		foregdcol = "#1E3C69";
	else
	if (config->rb_custom_checked)
		foregdcol = config->color_fg;

	if (!config->rb_sysdef_checked)
		pal.setColor(QPalette::Foreground, QColor(foregdcol)); // text

	lb_title_topleft->setPalette(pal);
	lb_title_topright->setPalette(pal);
	lb_pliststats->setPalette(pal);
	lb_dbasestats->setPalette(pal);
}

void qm_tracks::set_local_features(bool enabled)
{
	if (enabled)
		cb_dbasemode->setItemIcon ( 5, QIcon(":/tr_foldate.png") );
	else
		cb_dbasemode->setItemIcon ( 5, QIcon(":/tr_foldate_disabled.png") );
}

void qm_tracks::set_menu_random(bool isitso)
{
	a_random->blockSignals(true);
	a_random->setChecked(isitso);
	a_random->blockSignals(false);
}

void qm_tracks::set_menu_repeat(bool isitso)
{	
	a_repeat->blockSignals(true);
	a_repeat->setChecked(isitso);
	a_repeat->blockSignals(false);
}

void qm_tracks::set_menu_single(bool isitso)
{	
	a_single->blockSignals(true);
	a_single->setChecked(isitso);
	a_single->blockSignals(false);
}

void qm_tracks::set_menu_consume(bool isitso)
{
	a_consume->blockSignals(true);
	a_consume->setChecked(isitso);
	a_consume->blockSignals(false);
}

void qm_tracks::set_menu_xfade(bool isitso)
{
	a_xfade->blockSignals(true);
	a_xfade->setChecked(isitso);
	a_xfade->blockSignals(false);
}


void qm_tracks::set_random()
{
	if (mpd_connected)
		mpdCom->set_random(a_random->isChecked());
	else
		set_menu_random(!a_random->isChecked());
}

void qm_tracks::set_repeat()
{
	if (mpd_connected)
		mpdCom->set_repeat(a_repeat->isChecked());
	else
		set_menu_repeat(!a_repeat->isChecked()); 
}


void qm_tracks::set_single()
{
	if (mpd_connected)
		mpdCom->set_single(a_single->isChecked());
	else
		set_menu_single(!a_single->isChecked());
}


void qm_tracks::set_consume()
{	
	if (mpd_connected)
		mpdCom->set_consume(a_consume->isChecked());
	else
		set_menu_consume(!a_consume->isChecked());
}

void qm_tracks::set_xfade()
{
	if (mpd_connected)
	{
		if (a_xfade->isChecked())
			mpdCom->set_xfade(config->xfade_time);
		else
			mpdCom->set_xfade(0);
	}
	else
		set_menu_xfade(!a_xfade->isChecked());
}

qm_tracks::~qm_tracks()
{}